//
// BspFile.c  
//
// no GL here -- Benny

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "BspFile.h"

static FILE* bspFile = NULL;
static long oldFP = 0;
static BSPDIRECTORY dir;
static char entityString[MAX_ENTITY_STR+1];


STATUS openBSP(char* filename)
{
	STATUS result;

	
	if(bspFile != NULL)					
		return FAILURE;

	bspFile = fopen(filename, "rb");
	
	if(bspFile != NULL)
	{
		result = getDirectory(&dir);
		if(result == SUCCESS)
			return SUCCESS;
		else
			return FAILURE;
	}
	else
		return FAILURE;
}

STATUS closeBSP(void)
{
	if(bspFile != NULL)
	{
		if(fclose(bspFile) == 0)
			return SUCCESS;
		else
			return FAILURE;
	}
	else
		return FAILURE;
}

STATUS getDirectory(BSPDIRECTORY* pDirectory)
{
	size_t count = 0;

	// if a bsp file has been opened
	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the beginning of the file
		fseek(bspFile, 0, SEEK_SET);
		// read the directory into the callers structure
		count = fread(pDirectory, sizeof(BSPDIRECTORY), 1, bspFile);
		// if the full directory was read
		if(count == 1)
		{
			// restore the old file pointer
			fseek(bspFile, oldFP, SEEK_SET);
			// and return success
			return SUCCESS;
		}
		// else return failure
		else
			return FAILURE;
	}
	// else return failure
	else
		return FAILURE;

}

STATUS getModel(BSPMODEL* pModel, short idxModel)
{
	size_t count = 0;

	// if a bsp file has been opened
	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the beginning of the model array
		fseek(bspFile, dir.models.offset + idxModel * sizeof(BSPMODEL), SEEK_SET);
		// read the model into the callers structure
		count = fread(pModel, sizeof(BSPMODEL), 1, bspFile);
		// if the full model was read
		if(count == 1)
		{
			// restore the old file pointer
			fseek(bspFile, oldFP, SEEK_SET);
			// and return success
			return SUCCESS;
		}
		// else return failure
		else
			return FAILURE;
	}
	// else return failure
	else
		return FAILURE;
}

STATUS getFace(BSPFACE* pFace, unsigned short idxFace)
{
	size_t count = 0;

	// if a bsp file has been opened
	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the face
		fseek(bspFile, dir.faces.offset + idxFace * sizeof(BSPFACE), SEEK_SET);
		// read the face into the callers structure
		count = fread(pFace, sizeof(BSPFACE), 1, bspFile);
		// if the full face was read
		if(count == 1)
		{
			// restore the old file pointer
			fseek(bspFile, oldFP, SEEK_SET);
			// and return success
			return SUCCESS;
		}
		// else return failure
		else
			return FAILURE;
	}
	// else return failure
	else
		return FAILURE;
}

unsigned short getNumFaces(void)
{
	return dir.faces.size / sizeof(BSPFACE);
}

STATUS getFaceArray(BSPFACE* faceArray)
{
	/////////////////////////////////////////////////////
	// NOTE: This function assumes that a suitable amount
	//       of memory has been allocated for the
	//       faceArray paramater.
	/////////////////////////////////////////////////////
	
	long count;
	unsigned short numFaces;

	// get the number of faces in the bsp file
	numFaces = getNumFaces();

	// save the current file pointer
	oldFP = ftell(bspFile);

	// set the file pointer to the beginning of the face array
	fseek(bspFile, dir.faces.offset, SEEK_SET);

	// read the faces into the callers array
	count = fread(faceArray, sizeof(BSPFACE), numFaces, bspFile);

	// verify that all the faces were read
	if(count != numFaces)
		return FAILURE;

	// restore the old file pointer
	fseek(bspFile, oldFP, SEEK_SET);

	return SUCCESS;
}

STATUS getEdge(BSPEDGE* pEdge, long idxEdge)
{
	size_t count = 0;

	// if a bsp file has been opened
	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the edge
		fseek(bspFile, dir.edges.offset + idxEdge * sizeof(BSPEDGE), SEEK_SET);
		// read the edge into the callers structure
		count = fread(pEdge, sizeof(BSPEDGE), 1, bspFile);
		// if the full edge was read
		if(count == 1)
		{
			// restore the old file pointer
			fseek(bspFile, oldFP, SEEK_SET);
			// and return success
			return SUCCESS;
		}
		// else return failure
		else
			return FAILURE;
	}
	// else return failure
	else
		return FAILURE;
}

long getNumEdges(void)
{
	return dir.edges.size / sizeof(BSPEDGE);
}

STATUS getEdgeArray(BSPEDGE* edgeArray)
{
	/////////////////////////////////////////////////////
	// NOTE: This function assumes that a suitable amount
	//       of memory has been allocated for the
	//       edgeArray paramater.
	/////////////////////////////////////////////////////

	long count;
	long numEdges;

	// get the number of edges in the bsp file
	numEdges = getNumEdges();

	// save the current file pointer
	oldFP = ftell(bspFile);

	// set the file pointer to the beginning of the edge array
	fseek(bspFile, dir.edges.offset, SEEK_SET);

	// read the edges into the callers array
	count = fread(edgeArray, sizeof(BSPEDGE), numEdges, bspFile);

	// verify that all the edges were read
	if(count != numEdges)
		return FAILURE;

	// restore the old file pointer
	fseek(bspFile, oldFP, SEEK_SET);

	return SUCCESS;
}

STATUS getFaceIndex(unsigned short* pidxFace, unsigned short idxFaceIndex)
{
	size_t count = 0;

	// if a bsp file has been opened
	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the face index
		fseek(bspFile, dir.lfaces.offset + idxFaceIndex * sizeof(short), SEEK_SET);
		// read the face index into the callers structure
		count = fread(pidxFace, sizeof(short), 1, bspFile);
		// if the full edge was read
		if(count == 1)
		{
			// restore the old file pointer
			fseek(bspFile, oldFP, SEEK_SET);
			// and return success
			return SUCCESS;
		}
		// else return failure
		else
			return FAILURE;
	}
	// else return failure
	else
		return FAILURE;
}

STATUS getEdgeIndex(long* pidxEdge, long idxEdgeIndex)
{
	size_t count = 0;

	// if a bsp file has been opened
	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the edge index
		fseek(bspFile, dir.ledges.offset + idxEdgeIndex * sizeof(long), SEEK_SET);
		// read the edge index into the callers structure
		count = fread(pidxEdge, sizeof(long), 1, bspFile);
		// if the full edge was read
		if(count == 1)
		{
			// restore the old file pointer
			fseek(bspFile, oldFP, SEEK_SET);
			// and return success
			return SUCCESS;
		}
		// else return failure
		else
			return FAILURE;
	}
	// else return failure
	else
		return FAILURE;
}

long getNumEdgeIndices(void)
{
	return dir.ledges.size / sizeof(long);
}

STATUS getEdgeIndicesArray(long* edgeIndicesArray)
{
	/////////////////////////////////////////////////////
	// NOTE: This function assumes that a suitable amount
	//       of memory has been allocated for the
	//       edgeIndicesArray paramater.
	/////////////////////////////////////////////////////

	long count;
	long numEdgeIndices;

	// get the number of edge indices in the bsp file
	numEdgeIndices = getNumEdgeIndices();

	// save the current file pointer
	oldFP = ftell(bspFile);

	// set the file pointer to the beginning of the edge indices array
	fseek(bspFile, dir.ledges.offset, SEEK_SET);

	// read the edge indices into the callers array
	count = fread(edgeIndicesArray, sizeof(long), numEdgeIndices, bspFile);

	// verify that all the edge indices were read
	if(count != numEdgeIndices)
		return FAILURE;

	// restore the old file pointer
	fseek(bspFile, oldFP, SEEK_SET);

	return SUCCESS;
}

STATUS getVertex(BSPVERTEX* pVertex, unsigned short idxVertex)
{
	size_t count = 0;

	// if a bsp file has been opened
	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the vertex
		fseek(bspFile, dir.vertices.offset + idxVertex * sizeof(BSPVERTEX), SEEK_SET);
		// read the vertex into the callers structure
		count = fread(pVertex, sizeof(BSPVERTEX), 1, bspFile);
		// if the full vertex was read
		if(count == 1)
		{
			// restore the old file pointer
			fseek(bspFile, oldFP, SEEK_SET);
			// and return success
			return SUCCESS;
		}
		// else return failure
		else
			return FAILURE;
	}
	// else return failure
	else
		return FAILURE;	
}

unsigned short getNumVertices(void)
{
	return dir.vertices.size / sizeof(BSPVERTEX);
}

STATUS getVerticesArray(BSPVERTEX* verticesArray)
{
	/////////////////////////////////////////////////////
	// NOTE: This function assumes that a suitable amount
	//       of memory has been allocated for the
	//       verticesArray paramater.
	/////////////////////////////////////////////////////

	long count;
	unsigned short numVertices;

	// get the number of vertices in the bsp file
	numVertices = getNumVertices();

	// save the current file pointer
	oldFP = ftell(bspFile);

	// set the file pointer to the beginning of the vertices array
	fseek(bspFile, dir.vertices.offset, SEEK_SET);

	// read the vertices into the callers array
	count = fread(verticesArray, sizeof(BSPVERTEX), numVertices, bspFile);

	// verify that all the vertices were read
	if(count != numVertices)
		return FAILURE;

	// restore the old file pointer
	fseek(bspFile, oldFP, SEEK_SET);

	return SUCCESS;
}

STATUS getPlayerStartCoords(BSPVERTEX* pCoords)
{
	
	char* p;
	size_t count = 0;
//	int x, y, z;

	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the start of the entity lump
		fseek(bspFile, dir.entities.offset, SEEK_SET);
		// read the entire entity string into the array
		count = fread(entityString, sizeof(char), dir.entities.size, bspFile);
		if(count != (unsigned int)dir.entities.size)
			return FAILURE;
		// add the null character at the end of the array
		entityString[MAX_ENTITY_STR] = '\0';
		// find the coords
		p = strstr(entityString, "info_player_start");
		while (*p-- != '{'); // backtrack a little
		p = strstr(p, "origin");
		p = strstr(p, "\"");
		p = strstr(p, "\"");
		p++;
		sscanf(p, "%f %f %f", &pCoords->x, &pCoords->y, &pCoords->z);
		//pCoords->z = (90.f - (float)z) * (float)0xFFFF / 360.f;

		return SUCCESS;
	}
	else
		return FAILURE;
}

/*STATUS getMiptexLumpHeader(BSPMIPTEXLUMPHEADER* pHeader)
{

}*/

STATUS getTexinfo(BSPTEXINFO* pTexinfo, short idxTexinfo)
{
	size_t count = 0;

	// if a bsp file has been opened
	if(bspFile != NULL)
	{
		// save the old file pointer
		oldFP = ftell(bspFile);
		// move the file pointer to the texinfo
		fseek(bspFile, dir.texinfo.offset + idxTexinfo * sizeof(BSPTEXINFO), SEEK_SET);
		// read the vertex into the callers structure
		count = fread(pTexinfo, sizeof(BSPTEXINFO), 1, bspFile);
		// if the full vertex was read
		if(count == 1)
		{
			// restore the old file pointer
			fseek(bspFile, oldFP, SEEK_SET);
			// and return success
			return SUCCESS;
		}
		// else return failure
		else
			return FAILURE;
	}
	// else return failure
	else
		return FAILURE;		
}

short getNumTexinfos(void)
{
	return dir.texinfo.size / sizeof(BSPTEXINFO);
}

STATUS getTexinfoArray(BSPTEXINFO* texinfoArray)
{
	/////////////////////////////////////////////////////
	// NOTE: This function assumes that a suitable amount
	//       of memory has been allocated for the
	//       texinfoArray paramater.
	/////////////////////////////////////////////////////

	long count;
	short numTexinfos;

	// get the number of texinfos in the bsp file
	numTexinfos = getNumTexinfos();

	// save the current file pointer
	oldFP = ftell(bspFile);

	// set the file pointer to the beginning of the texinfo array
	fseek(bspFile, dir.texinfo.offset, SEEK_SET);

	// read the texinfos into the callers array
	count = fread(texinfoArray, sizeof(BSPTEXINFO), numTexinfos, bspFile);

	// verify that all the vertices were read
	if(count != numTexinfos)
		return FAILURE;

	// restore the old file pointer
	fseek(bspFile, oldFP, SEEK_SET);

	return SUCCESS;
}

long getSizeOfMiptexLump(void)
{
	return dir.miptex.size;
}

STATUS getMiptexLump(BSPMIPTEXLUMP* pMiptexLump)
{
	/////////////////////////////////////////////////////
	// NOTE: This function assumes that a suitable amount
	//       of memory has been allocated for the
	//       miptexLump paramater.
	/////////////////////////////////////////////////////

	long count;
	long sizeOfMiptexLump;

	// get the size of the miptex lump
	sizeOfMiptexLump = getSizeOfMiptexLump();

	// save the current file pointer
	oldFP = ftell(bspFile);

	// set the file pointer to the beginning of the miptex lump
	fseek(bspFile, dir.miptex.offset, SEEK_SET);

	// read the miptex lump into the callers array
	count = fread(pMiptexLump, sizeOfMiptexLump, 1, bspFile);

	// verify that the lump was read
	if(count != 1)
		return FAILURE;

	// restore the old file pointer
	fseek(bspFile, oldFP, SEEK_SET);

	return SUCCESS;
}
	 
