/*
 
 leveldata.c                                              

 This module contains routines for loading the necessary  
 information from a Quake BSP file. 

*/

// no GL here -- Benny

#ifdef _WIN32
 #include <windows.h>
 #define error(a) 	MessageBox(0, (a), 0, MB_OK);
#else 
 void quit(char *);
 #define error 		quit
#endif

#include <stdlib.h>
#include <time.h>

#include "Bspfile.h"

float randUnitScalar(void); 

BSPMODEL levelModel;

BSPPALETTEENTRY qpalette[256];

BSPFACE* faces = NULL;
unsigned short numFaces = 0;
/*
BSPTEXINFO* texinfos = NULL;
short numTexinfos = 0;

BSPMIPTEXLUMP* miptexlump = NULL;
long sizeOfMiptexLump = 0;
*/
BSPEDGE* edges = NULL;
long numEdges = 0;

long* edgeIndices = NULL;
long numEdgeIndices = 0;

BSPVERTEX* vertices = NULL;
unsigned short numVertices = 0;

BSPVERTEX* v = NULL; 

/*---------------------------------------------------------------*/

STATUS loadLevelData(void)
{
	int i;

	getModel(&levelModel, 0);
	
	numFaces = getNumFaces();
	faces = (BSPFACE*)calloc(numFaces, sizeof(BSPFACE));
	if(faces == NULL)
	{
		error("Error allocating faces array.");
		return FAILURE;
	}
	getFaceArray(faces);
	v = (BSPVERTEX*)calloc(numFaces, sizeof(BSPVERTEX));
	if(v == NULL)
	{
		error("Error allocating face color array.");
		return FAILURE;
	}
	for(i=0; i<numFaces; i++)
	{
		v[i].x = randUnitScalar();
		v[i].y = randUnitScalar();
		v[i].z = randUnitScalar();
	}
/*
	numTexinfos = getNumTexinfos();
	texinfos = (BSPTEXINFO*)calloc(numTexinfos, sizeof(BSPTEXINFO));
	if(texinfos == NULL)
	{
		error("Error allocating texinfos array.");
		return FAILURE;
	}
	getTexinfoArray(texinfos);
	
	sizeOfMiptexLump = getSizeOfMiptexLump();
	miptexlump = (BSPMIPTEXLUMP*)calloc(1, sizeOfMiptexLump);
	if(miptexlump == NULL)
	{
		error("Error allocating miptex lump.");
		return FAILURE;
	}
	getMiptexLump(miptexlump); 
*/
		
	numEdges = getNumEdges();
	edges = (BSPEDGE*)calloc(numEdges, sizeof(BSPEDGE));
	if(edges == NULL)
	{
		error("Error allocating edges array.");
		return FAILURE;
	}
	getEdgeArray(edges);

	numEdgeIndices = getNumEdgeIndices();
	edgeIndices = (long*)calloc(numEdgeIndices, sizeof(long));
	if(edgeIndices == NULL)
	{
		error("Error allocating edge indices array.");
		return FAILURE;
	}
	getEdgeIndicesArray(edgeIndices);

	numVertices = getNumVertices();
	vertices = (BSPVERTEX*)calloc(numVertices, sizeof(BSPVERTEX));
	if(vertices == NULL)
	{
		error("Error allocating vertices array.");
		return FAILURE;
	}
	getVerticesArray(vertices);

	return SUCCESS;
} 

float randUnitScalar(void)
{     
    return ((float)rand()) / ((float)(RAND_MAX+1));     
} 

void destroyLevelData(void)
{
	if(faces != NULL)
		free(faces);
/*
	if(texinfos != NULL)
		free(texinfos);
	if(miptexlump != NULL)
		free(miptexlump);
*/
	if(edges != NULL)
		free(edges);
	if(edgeIndices != NULL)
		free(edgeIndices);
	if(vertices != NULL)
		free(vertices);
}
