/*---------------------------------------------------------\

Experiments with 3dfx OpenGL,
using QGL (Jan 97) from Joel Beaudoin.
 
Collected GL stuff here:
QGLmain = oglrc.c + camera.c + view.c + qgl.c  

The program looks for BSPFILE in the directory BSPPOS.
Then it displays the level with color-filled polygons.
You can navigate around with the keypad keys.

I don't pretend to understand all of it, but it seems to work ;)

Note: if you don't have files like start.bsp or e1m1.bsp,
you must un-pak the Quake pak0.pak file with an unpak utility
(available on most Quake servers in the tools or the unpak
directory.)

-- Benny.
 
\---------------------------------------------------------*/

// You may want to change the location here
#define BSPPOS				"/QUAKE/ID1/MAPS/"

// You may want to change the bsp name here
#define BSPFILE BSPPOS 		"start.bsp" //"E1M1.BSP"

/*---------------------------------------------------------*/

#include <windows.h>

/*-start gl-*/

#include <gl\gl.h>
#include <gl\glu.h>

void SetupGL()
{
	glEnable(GL_DEPTH_TEST);	// Hidden surface removal
	glFrontFace(GL_CW);		    // Counter clock-wise polygons face out
	glEnable(GL_CULL_FACE);		// Do not calculate inside of jet
	glPolygonMode(GL_FRONT, GL_FILL); // Filled Poly
}

void moveCamera(float dx, float dy, float dz)
{
	GLfloat m[4][4];

	glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat*)m);
	glLoadIdentity();
	glTranslatef(-dx, -dy, -dz);
	glMultMatrixf((GLfloat*)m);
}

void turnCamera(float angle)
{
	GLfloat m[4][4];

	glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat*)m);
	glLoadIdentity();
	glRotatef(-angle, 0.0f, 1.0f, 0.0f);
	glMultMatrixf((GLfloat*)m);
}
 
void tiltCamera(float angle)
{
	GLfloat m[4][4];

	glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat*)m);
	glLoadIdentity();
	glRotatef(-angle, 1.0f, 0.0f, 0.0f);
	glMultMatrixf((GLfloat*)m);
}
 
void rollCamera(float angle)
{
	GLfloat m[4][4];

	glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat*)m);
	glLoadIdentity();
	glRotatef(-angle, 0.0f, 0.0f, 1.0f);
	glMultMatrixf((GLfloat*)m);
} 
 
#include "BspStruc.h"
#include "leveldat.h"

void ChangeSize()
{
	GLfloat fAspect;
	GLsizei w1 = 640;
	GLsizei h1 = 480;

	// Set Viewport to window dimensions
    glViewport(0, 0, w1, h1);

	fAspect = (GLfloat)w1/(GLfloat)h1;

	// Reset coordinate system
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Produce the perspective projection
	gluPerspective(60.0f, fAspect, 1.0, 10000.0);

	glMatrixMode(GL_MODELVIEW);
}

void RenderScene(void)
{
	
	BSPFACE* pCurFace = NULL;
	BSPEDGE* pCurEdge = NULL;
	BSPVERTEX* pCurVertex = NULL;
	long edgeIndex;
	int i, j;

	// Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	for(i=0; i<levelModel.numFaces; i++)
	{
		pCurFace = &faces[levelModel.idxFaces + i];
		glColor3f(v[i].x, v[i].y, v[i].z);
		glBegin(GL_POLYGON);
			for(j=0; j<pCurFace->numEdges; j++)
			{
				edgeIndex = edgeIndices[pCurFace->idxEdges + j];
				if(edgeIndex > 0)
				{
					pCurEdge = &edges[edgeIndex];
					pCurVertex = &vertices[pCurEdge->idxV0];
				}
				else
				{
					pCurEdge = &edges[-edgeIndex];
					pCurVertex = &vertices[pCurEdge->idxV1];
				}
				//These functions specify a vertex.
				glVertex3f(pCurVertex->x, pCurVertex->z, -pCurVertex->y);
			}
		glEnd();
	}	 
}

void setupGL_Player()
{
	BSPVERTEX pCoords;	 
	
	pCoords.x = 544.f;
	pCoords.y = 288.f;
	pCoords.z = 32.f;	 
	
	glTranslatef(-pCoords.x, -pCoords.z, pCoords.y);	 	 
}
 
// Win stuff
static HGLRC hRC; 
static HDC hDC; 

void deselect()
{
	wglMakeCurrent(hDC,NULL);
	wglDeleteContext(hRC);
}

/*-gl end-*/

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM  wParam, LPARAM lParam);
void handleCreate(HWND hWnd);
void handleDestroy(void);
void handleChar(HWND hWnd, WPARAM wParam);
void handlePaint(HWND hWnd);
 

int APIENTRY WinMain(HINSTANCE hInst,
					 HINSTANCE hPrevInstance,
					 LPSTR lpCmdLine,
					 int nCmdShow)
{
	MSG msg;           
	WNDCLASS wc;                    
	HWND hWnd;           

	wc.style                = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc          = (WNDPROC) WndProc;
	wc.cbClsExtra           = 0;
	wc.cbWndExtra           = 0;
	wc.hInstance            = hInst;
	wc.hIcon                = NULL;
	wc.hCursor              = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground        = (HBRUSH)COLOR_ACTIVECAPTION;//COLOR_GRAYTEXT; 
	wc.lpszMenuName         = NULL;
	wc.lpszClassName        = "Qgl";

	if(RegisterClass(&wc) == 0)
		return FALSE;

	hWnd = CreateWindow(
				"Qgl",
				"Benny's Qgl: Use the Key Pad to move around.",
				WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
				0, 0,
				640, 480,
				NULL,
				NULL,
				hInst,
				NULL);

	if(hWnd == NULL)
		return FALSE;

	ShowWindow(hWnd,SW_SHOW);
	UpdateWindow(hWnd);

	while(GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM  wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_CREATE:
			handleCreate(hWnd);			 
			break;

		case WM_DESTROY:
			handleDestroy();
			break;

		case WM_CHAR:
			handleChar(hWnd, wParam);
			break;

		case WM_SIZE:
			ChangeSize();
			break;

		case WM_PAINT:
			handlePaint(hWnd);
			break;
 
		default:   
			return (DefWindowProc(hWnd, message, wParam, lParam));
	}

    return (0L);
}

void checkPalette(HDC hDC)
{
	PIXELFORMATDESCRIPTOR pfd;	// Pixel Format Descriptor
    int nPixelFormat;               // Pixel format index

	// Get the pixel format index and retrieve the pixel format description
	nPixelFormat = GetPixelFormat(hDC);
    DescribePixelFormat(hDC, nPixelFormat,
                sizeof(PIXELFORMATDESCRIPTOR), &pfd);

    // Does this pixel format require a palette?
    // If not, do not create a palette and just return
	if(!(pfd.dwFlags & PFD_NEED_PALETTE))
        return;

	// Well, I took out the palette stuff - Benny.
    MessageBox(NULL, "Error: this window wants a palette! (no more implemented)", NULL, MB_OK);
	handleDestroy();     
}

void SetDCPixelFormat(HDC hDC)
{
	int nPixelFormat;

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),			// Size of this structure
		1,                                      // Version of this structure    
		PFD_DRAW_TO_WINDOW |                    // Draw to Window (not to bitmap)
		PFD_SUPPORT_OPENGL |					// Support OpenGL calls in window
		PFD_DOUBLEBUFFER,                       // Double buffered
		PFD_TYPE_RGBA,                          // RGBA Color mode
		24,                                     // Want 24bit color 
		0,0,0,0,0,0,                            // Not used to select mode
		0,0,                                    // Not used to select mode
		0,0,0,0,0,                              // Not used to select mode
		32,                                     // Size of depth buffer
		0,                                      // Not used to select mode
		0,                                      // Not used to select mode
		PFD_MAIN_PLANE,                         // Draw in main plane
		0,                                      // Not used to select mode
		0,0,0 };                                // Not used to select mode

	// Choose a pixel format that best matches that described in pfd
	nPixelFormat = ChoosePixelFormat(hDC, &pfd);

	// Set the pixel format for the device context
	SetPixelFormat(hDC, nPixelFormat, &pfd);
}

 
STATUS openBSP(char* filename);
 
void handleCreate(HWND hWnd)
{
	static rval;

	// Store the device context
	hDC = GetDC(hWnd);              

	// Select the pixel format
	SetDCPixelFormat(hDC);          

	checkPalette(hDC);

	// Create the rendering context and make it current
	hRC = wglCreateContext(hDC);
	wglMakeCurrent(hDC, hRC);

	SetupGL();  

	rval = openBSP(BSPFILE);

	if(rval == FAILURE)
	{
		MessageBox(NULL, "Error opening bsp file (not found?)", NULL, MB_OK);
		exit(1);
	}
	 
 	loadLevelData();
	setupGL_Player();			
}

void handleDestroy()
{	 	 
	deselect();
	destroyLevelData();	 
	PostQuitMessage(0);
}

void handlePaint(HWND hWnd)
{
	// Call OpenGL drawing code
	RenderScene(); 

	SwapBuffers(hDC);	 
	ValidateRect(hWnd,NULL);
}

void handleChar(HWND hWnd, WPARAM wParam)
{
	RECT r;
	
	switch(toupper(wParam))
	{
 
		case '8':		 
			moveCamera(0.0f, 0.0f, -20.0f);
			GetClientRect(hWnd, &r);
			InvalidateRect(hWnd, &r, FALSE);
			break;
 
		case '2':		 
			moveCamera(0.0f, 0.0f, 20.0f);
			GetClientRect(hWnd, &r);
			InvalidateRect(hWnd, &r, FALSE);
			break;	
 
		case '3':
			moveCamera(-20.0f, 0.0f, 0.0f);
			GetClientRect(hWnd, &r);
			InvalidateRect(hWnd, &r, FALSE);
			break;	
 
		case '1':
			moveCamera(20.0f, 0.0f, 0.0f);
			GetClientRect(hWnd, &r);
			InvalidateRect(hWnd, &r, FALSE);
			break;		
		 
		case '-':
		case '0':
			moveCamera(0.0f, -20.0f, 0.0f);
			GetClientRect(hWnd, &r);
			InvalidateRect(hWnd, &r, FALSE);
			break;
		
		case '+':
		case '.':
			moveCamera(0.0f, 20.0f, 0.0f);
			GetClientRect(hWnd, &r); InvalidateRect(hWnd, &r, FALSE);
			break;
	 
		case '4':
			turnCamera(10.0f);
			GetClientRect(hWnd, &r);
			InvalidateRect(hWnd, &r, FALSE);
			break;
	 
		case '6':
			turnCamera(-10.0f);
			GetClientRect(hWnd, &r);
			InvalidateRect(hWnd, &r, FALSE);
			break;

		case 0x1b:			
			handleDestroy();			 
			break;

		case 'W': // wireframe
		case '7':
			glPolygonMode(GL_FRONT, GL_LINE);
			GetClientRect(hWnd, &r); InvalidateRect(hWnd, &r, FALSE);
			break;

		case 'F': // filled
		case '9':
			glPolygonMode(GL_FRONT, GL_FILL);
			GetClientRect(hWnd, &r); InvalidateRect(hWnd, &r, FALSE);
			break;
 	}
} 
/*-eof-*/
